# -*- coding: utf-8 -*-
import asyncio
import traceback
from discord.ext.voice_recv import BasicSink
import discord.ext.voice_recv
import discord
from discord import app_commands, FFmpegPCMAudio, AudioSource
from discord.ext.voice_recv.sinks import AudioSink
from discord.ext import commands, tasks, voice_recv
from typing import Optional, Dict, Set
import sqlite3
import logging
from datetime import datetime, timedelta, timezone
import json
import google.generativeai as genai
from google.generativeai.types import HarmCategory, HarmBlockThreshold
import requests
from bs4 import BeautifulSoup
import time
import re
import pytz
from collections import defaultdict
import logging
try:
    from .commands import *
except ImportError:
    pass

import queue
import threading
from nana_bot import (
    bot,
    bot_name,
    WHITELISTED_SERVERS,
    TARGET_CHANNEL_ID,
    API_KEY,
    init_db,
    gemini_model,
    servers,
    send_daily_channel_id_list,
    not_reviewed_id,
    newcomer_channel_id,
    welcome_channel_id,
    member_remove_channel_id,
    discord_bot_token,
    review_format,
    debug,
    Point_deduction_system,
    default_points
)
import os
import numpy as np
import torch
import torchaudio
import whisper


import tempfile
import edge_tts
import functools
import wave
import uuid
import io


whisper_model = None
vad_model = None

VAD_SAMPLE_RATE = 16000
VAD_EXPECTED_SAMPLES = 512
VAD_CHUNK_SIZE_BYTES = VAD_EXPECTED_SAMPLES * 2
VAD_THRESHOLD = 0.5
VAD_MIN_SILENCE_DURATION_MS = 700
VAD_SPEECH_PAD_MS = 200


audio_buffers = defaultdict(lambda: {
    'buffer': bytearray(),
    'pre_buffer': bytearray(),
    'last_speech_time': time.time(),
    'is_speaking': False
})

listening_guilds: Dict[int, discord.VoiceClient] = {}
voice_clients: Dict[int, discord.VoiceClient] = {}

expecting_voice_query_from: Set[int] = set()
QUERY_TIMEOUT_SECONDS = 30


safety_settings = {
    HarmCategory.HARM_CATEGORY_HATE_SPEECH:      HarmBlockThreshold.BLOCK_NONE,
    HarmCategory.HARM_CATEGORY_HARASSMENT:       HarmBlockThreshold.BLOCK_NONE,
    HarmCategory.HARM_CATEGORY_SEXUALLY_EXPLICIT: HarmBlockThreshold.BLOCK_NONE,
    HarmCategory.HARM_CATEGORY_DANGEROUS_CONTENT: HarmBlockThreshold.BLOCK_NONE,
}

DEFAULT_VOICE = "zh-TW-HsiaoYuNeural"
STT_LANGUAGE = "zh"

logging.basicConfig(level=logging.INFO if not debug else logging.DEBUG,
                    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s',
                    handlers=[
                        logging.FileHandler("bot.log", encoding='utf-8'),
                        logging.StreamHandler()
                    ])
logger = logging.getLogger(__name__)
discord_logger = logging.getLogger('discord')
discord_logger.setLevel(logging.WARNING)


async def play_tts(voice_client: discord.VoiceClient, text: str, context: str = "TTS"):
    total_start = time.time()
    if not voice_client or not voice_client.is_connected():
        logger.warning(f"[{context}] 無效或未連接的 voice_client，無法播放 TTS: '{text}'")
        return

    logger.info(f"[{context}] 開始為文字生成 TTS: '{text[:50]}...' (Guild: {voice_client.guild.id})")
    loop = asyncio.get_running_loop()
    tmp_path = None
    source = None
    playback_started = False

    try:
        step1 = time.time()
        communicate = edge_tts.Communicate(text, DEFAULT_VOICE)
        with tempfile.NamedTemporaryFile(suffix=".mp3", delete=False) as tmp:
            tmp_path = tmp.name
        logger.debug(f"[{context}] 暫存檔案路徑: {tmp_path}")

        await loop.run_in_executor(None, functools.partial(communicate.save_sync, tmp_path))
        logger.info(f"[{context}] 步驟 1 (生成音檔) 耗時 {time.time()-step1:.4f}s -> {tmp_path}")

        step2 = time.time()
        ffmpeg_options = {
            'before_options': '',
            'options': '-vn'
        }
        if not os.path.exists(tmp_path):
             logger.error(f"[{context}] 暫存檔案 {tmp_path} 在創建音源前消失了！")
             return

        source = await loop.run_in_executor(
            None,
            lambda: FFmpegPCMAudio(tmp_path, **ffmpeg_options)
        )
        logger.info(f"[{context}] 步驟 2 (創建音源) 耗時 {time.time()-step2:.4f}s")

        if not voice_client.is_connected():
             logger.warning(f"[{context}] 創建音源後，語音客戶端已斷開連接。")
             if tmp_path and os.path.exists(tmp_path):
                 try:
                     os.remove(tmp_path)
                     logger.info(f"[{context}][Cleanup] 已清理因斷線未播放的暫存檔案: {tmp_path}")
                 except OSError as e:
                     logger.warning(f"[{context}][Cleanup] 清理未播放暫存檔 {tmp_path} 失敗: {e}")
             return

        if voice_client.is_playing():
            logger.info(f"[{context}] 停止當前播放以播放新的 TTS。")
            voice_client.stop()
            await asyncio.sleep(0.1)

        step3 = time.time()
        def _cleanup(error, path_to_clean):
            log_prefix = f"[{context}][Cleanup]"
            if error:
                logger.error(f"{log_prefix} 播放器錯誤: {error}")
            else:
                 logger.info(f"{log_prefix} TTS 播放完成。")
            try:
                if path_to_clean and os.path.exists(path_to_clean):
                    os.remove(path_to_clean)
                    logger.info(f"{log_prefix} 已清理暫存檔案: {path_to_clean}")
            except OSError as e:
                logger.warning(f"{log_prefix} 清理暫存檔案 {path_to_clean} 失敗: {e}")
            except Exception as cleanup_err:
                 logger.error(f"{log_prefix} 清理檔案時發生錯誤: {cleanup_err}")

        voice_client.play(source, after=lambda e, p=tmp_path: _cleanup(e, p))
        playback_started = True
        logger.info(f"[{context}] 步驟 3 (開始播放) 耗時 {time.time()-step3:.4f}s (背景執行)")
        logger.info(f"[{context}] 從請求到開始播放總耗時: {time.time()-total_start:.4f}s")

    except edge_tts.NoAudioReceived:
        logger.error(f"[{context}] Edge TTS 失敗: 未收到音檔。 文字: '{text[:50]}...'")
    except edge_tts.exceptions.UnexpectedStatusCode as e:
         logger.error(f"[{context}] Edge TTS 失敗: 非預期狀態碼 {e.status_code}。 文字: '{text[:50]}...'")
    except FileNotFoundError:
        logger.error(f"[{context}] FFmpeg 錯誤: 找不到 FFmpeg 執行檔。請確保 FFmpeg 已安裝並在系統 PATH 中。")
    except discord.errors.ClientException as e:
        logger.error(f"[{context}] Discord 客戶端錯誤 (播放時): {e}")
    except Exception as e:
        logger.exception(f"[{context}] play_tts 中發生非預期錯誤。 文字: '{text[:50]}...'")

    finally:
        if not playback_started and tmp_path and os.path.exists(tmp_path):
            logger.warning(f"[{context}][Finally] 播放未成功開始，清理暫存檔案: {tmp_path}")
            try:
                os.remove(tmp_path)
            except OSError as e:
                logger.warning(f"[{context}][Finally] 清理未播放的暫存檔案 {tmp_path} 失敗: {e}")
            except Exception as final_e:
                 logger.error(f"[{context}][Finally] 清理未播放檔案時發生錯誤: {final_e}")


def get_current_time_utc8():
    utc8 = timezone(timedelta(hours=8))
    current_time = datetime.now(utc8)
    return current_time.strftime("%Y-%m-%d %H:%M:%S")

genai.configure(api_key=API_KEY)
not_reviewed_role_id = not_reviewed_id
try:
    if not API_KEY:
        raise ValueError("Gemini API key is not set.")
    model = genai.GenerativeModel(gemini_model)
    logger.info(f"成功初始化 GenerativeModel: {gemini_model}")
except Exception as e:
    logger.critical(f"初始化 GenerativeModel 失敗: {e}")
    model = None

db_base_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), "databases")
os.makedirs(db_base_path, exist_ok=True)

def get_db_path(guild_id, db_type):
    if db_type == 'analytics':
        return os.path.join(db_base_path, f"analytics_server_{guild_id}.db")
    elif db_type == 'chat':
        return os.path.join(db_base_path, f"messages_chat_{guild_id}.db")
    elif db_type == 'points':
        return os.path.join(db_base_path, f"points_{guild_id}.db")
    else:
        raise ValueError(f"Unknown database type: {db_type}")

def init_db_for_guild(guild_id):
    logger.info(f"正在為伺服器 {guild_id} 初始化資料庫...")
    db_tables = {
        "users": "user_id TEXT PRIMARY KEY, user_name TEXT, join_date TEXT, message_count INTEGER DEFAULT 0",
        "messages": "message_id INTEGER PRIMARY KEY AUTOINCREMENT, user_id TEXT, user_name TEXT, channel_id TEXT, timestamp TEXT, content TEXT",
        "metadata": """id INTEGER PRIMARY KEY AUTOINCREMENT,
                    userid TEXT UNIQUE,
                    total_token_count INTEGER,
                    channelid TEXT""",
        "reviews": """review_id INTEGER PRIMARY KEY AUTOINCREMENT,
                    user_id TEXT,
                    review_date TEXT"""
    }
    points_tables = {
        "users": f"user_id TEXT PRIMARY KEY, user_name TEXT, join_date TEXT, points INTEGER DEFAULT {default_points}",
        "transactions": f"id INTEGER PRIMARY KEY AUTOINCREMENT, user_id TEXT, points INTEGER, reason TEXT, timestamp TEXT"
    }
    chat_tables = {
         "message": "id INTEGER PRIMARY KEY AUTOINCREMENT, user TEXT, content TEXT, timestamp TEXT"
    }

    def _init_single_db(db_path, tables_dict):
        conn = None
        try:
            conn = sqlite3.connect(db_path, timeout=10)
            cursor = conn.cursor()
            for table_name, table_schema in tables_dict.items():
                cursor.execute(f"CREATE TABLE IF NOT EXISTS {table_name} ({table_schema})")
            conn.commit()
            logger.debug(f"資料庫已初始化/檢查: {db_path}")
        except sqlite3.OperationalError as e:
             logger.error(f"初始化資料庫 {db_path} 時發生 OperationalError (可能是權限或路徑問題): {e}")
        except sqlite3.Error as e:
            logger.exception(f"初始化資料庫 {db_path} 時發生錯誤: {e}")
        finally:
            if conn:
                conn.close()

    _init_single_db(get_db_path(guild_id, 'analytics'), db_tables)
    _init_single_db(get_db_path(guild_id, 'points'), points_tables)
    _init_single_db(get_db_path(guild_id, 'chat'), chat_tables)
    logger.info(f"伺服器 {guild_id} 的資料庫初始化完成。")



@tasks.loop(hours=24)
async def send_daily_message():
    logger.info("開始執行每日訊息任務...")
    for idx, server_id in enumerate(servers):
        try:
            if idx < len(send_daily_channel_id_list) and idx < len(not_reviewed_id):
                target_channel_id = send_daily_channel_id_list[idx]
                role_to_mention_id = not_reviewed_id[idx]
                channel = bot.get_channel(target_channel_id)
                guild = bot.get_guild(server_id)

                if channel and guild:
                    role = guild.get_role(role_to_mention_id)
                    if role:
                        try:
                            await channel.send(
                                f"{role.mention} 各位未審核的人，快來這邊審核喔"
                            )
                            logger.info(f"Sent daily message to channel {target_channel_id} in guild {server_id}")
                        except discord.Forbidden:
                            logger.error(f"Permission error sending daily message to channel {target_channel_id} in guild {server_id}.")
                        except discord.HTTPException as e:
                            logger.error(f"HTTP error sending daily message to channel {target_channel_id} in guild {server_id}: {e}")
                    else:
                        logger.warning(f"Role {role_to_mention_id} not found in guild {server_id} for daily message.")
                else:
                    if not channel:
                        logger.warning(f"Daily message channel {target_channel_id} not found for server index {idx} (Guild ID: {server_id}).")
                    if not guild:
                         logger.warning(f"Guild {server_id} not found for daily message.")
            else:
                logger.error(f"Configuration index {idx} out of range for daily message (Guild ID: {server_id}). Lists length: send_daily={len(send_daily_channel_id_list)}, not_reviewed={len(not_reviewed_id)}")
        except Exception as e:
            logger.exception(f"Unexpected error in send_daily_message loop for server index {idx} (Guild ID: {server_id}): {e}")
    logger.info("每日訊息任務執行完畢。")


@send_daily_message.before_loop
async def before_send_daily_message():
    await bot.wait_until_ready()
    now = datetime.now(pytz.timezone('Asia/Taipei'))
    next_run = now.replace(hour=9, minute=0, second=0, microsecond=0)
    if next_run < now:
        next_run += timedelta(days=1)
    wait_seconds = (next_run - now).total_seconds()
    logger.info(f"每日訊息任務將在 {wait_seconds:.0f} 秒後首次執行 (於 {next_run.strftime('%Y-%m-%d %H:%M:%S %Z')})")
    await asyncio.sleep(wait_seconds)


@bot.event
async def on_ready():
    logger.info(f"以 {bot.user.name} (ID: {bot.user.id}) 登入")
    logger.info(f"Discord.py 版本: {discord.__version__}")
    logger.info("機器人已準備就緒並連接到 Discord。")

    if model is None:
        logger.error("AI 模型初始化失敗。AI 回覆功能將被禁用。")

    guild_count = 0
    for guild in bot.guilds:
        guild_count += 1
        logger.info(f"機器人所在伺服器: {guild.name} (ID: {guild.id})")
        init_db_for_guild(guild.id)

    logger.info("正在同步應用程式命令...")
    synced_commands = 0
    try:

        for guild in bot.guilds:
             try:
                 synced = await bot.tree.sync()
                 synced_commands += len(synced)
                 logger.debug(f"已為伺服器 {guild.id} ({guild.name}) 同步 {len(synced)} 個命令。")
             except discord.errors.Forbidden:
                 logger.warning(f"無法為伺服器 {guild.id} ({guild.name}) 同步命令 (權限不足)。")
             except discord.HTTPException as e:
                 logger.error(f"為伺服器 {guild.id} ({guild.name}) 同步命令時發生 HTTP 錯誤: {e}")
        logger.info(f"總共同步了 {synced_commands} 個應用程式命令。")

    except discord.errors.Forbidden as e:
        logger.warning(f"因權限問題無法同步命令: {e}")
    except discord.HTTPException as e:
        logger.error(f"同步命令時發生 HTTP 錯誤: {e}")
    except Exception as e:
        logger.exception(f"同步命令時發生非預期錯誤: {e}")

    if not send_daily_message.is_running():
        send_daily_message.start()
        logger.info("已啟動每日訊息任務。")

    activity = discord.Game(name=f"在 {guild_count} 個伺服器上運作 | /help")
    await bot.change_presence(status=discord.Status.online, activity=activity)
    logger.info(f"機器人狀態已設定。正在監看 {guild_count} 個伺服器。")


@bot.event
async def on_guild_join(guild):
    logger.info(f"機器人加入新伺服器: {guild.name} (ID: {guild.id})")
    init_db_for_guild(guild.id)
    if guild.id not in servers:
        logger.warning(f"伺服器 {guild.id} ({guild.name}) 不在設定檔 'servers' 列表中。可能需要手動設定相關功能。")

    logger.info(f"正在為新伺服器 {guild.id} 同步命令...")
    try:
        synced = await bot.tree.sync(guild=guild)
        logger.info(f"已為新伺服器 {guild.id} ({guild.name}) 同步 {len(synced)} 個命令。")
    except discord.errors.Forbidden:
         logger.error(f"為新伺服器 {guild.id} ({guild.name}) 同步命令時權限不足。")
    except Exception as e:
         logger.exception(f"為新伺服器 {guild.id} ({guild.name}) 同步命令時出錯: {e}")

    channel_to_send = guild.system_channel or next((tc for tc in guild.text_channels if tc.permissions_for(guild.me).send_messages), None)
    if channel_to_send:
        try:
            await channel_to_send.send(f"大家好！我是 {bot_name}。很高興加入 **{guild.name}**！\n"
                                       f"您可以使用 `/help` 來查看我的指令。\n"
                                       f"如果想在語音頻道與我對話，請先使用 `/join` 加入，然後使用 `/ask_voice` 開始提問。\n"
                                       f"請確保已根據需求設定相關頻道 ID 和權限。\n"
                                       f"我的設定檔可能需要手動更新以包含此伺服器 ID ({guild.id}) 的相關設定 (例如審核頻道、歡迎頻道等)。")
            logger.info(f"已在伺服器 {guild.id} 的頻道 {channel_to_send.name} 發送歡迎訊息。")
        except discord.Forbidden:
            logger.warning(f"無法在伺服器 {guild.id} 的頻道 {channel_to_send.name} 發送歡迎訊息 (權限不足)。")
        except discord.HTTPException as e:
            logger.error(f"在伺服器 {guild.id} 的頻道 {channel_to_send.name} 發送歡迎訊息時發生 HTTP 錯誤: {e}")
    else:
        logger.warning(f"在伺服器 {guild.id} ({guild.name}) 中找不到適合發送歡迎訊息的頻道或缺少發送權限。")

@bot.event
async def on_member_join(member):
    guild = member.guild
    logger.info(f"新成員加入: {member} (ID: {member.id}) 於伺服器 {guild.name} (ID: {guild.id})")

    server_index = -1
    for idx, s_id in enumerate(servers):
        if guild.id == s_id:
            server_index = idx
            break

    if server_index == -1:
        logger.warning(f"No configuration found for server ID {guild.id} ({guild.name}) in on_member_join. Skipping role/welcome message.")
        analytics_db_path = get_db_path(guild.id, 'analytics')
        conn_user_join = None
        try:
            conn_user_join = sqlite3.connect(analytics_db_path, timeout=10)
            c_user_join = conn_user_join.cursor()
            join_utc = member.joined_at.astimezone(timezone.utc) if member.joined_at else datetime.now(timezone.utc)
            join_iso = join_utc.isoformat()
            c_user_join.execute(
                "INSERT OR IGNORE INTO users (user_id, user_name, join_date, message_count) VALUES (?, ?, ?, ?)",
                (str(member.id), member.name, join_iso, 0),
            )
            conn_user_join.commit()
            logger.debug(f"User {member.id} added/ignored in analytics DB for guild {guild.id}")
        except sqlite3.Error as e:
            logger.exception(f"Database error on member join (analytics) for guild {guild.id}: {e}")
        finally:
            if conn_user_join:
                conn_user_join.close()
        return

    try:
        current_welcome_channel_id = welcome_channel_id[server_index]
        current_role_id = not_reviewed_id[server_index]
        current_newcomer_channel_id = newcomer_channel_id[server_index]
    except IndexError:
        logger.error(f"Configuration index {server_index} out of range for server ID {guild.id}. Check config lists length (welcome_channel_id, not_reviewed_id, newcomer_channel_id).")
        return
    except NameError as e:
        logger.error(f"Configuration variable name error for server {guild.id}: {e}. Ensure lists are imported.")
        return


    analytics_db_path = get_db_path(guild.id, 'analytics')
    conn_user_join = None
    try:
        conn_user_join = sqlite3.connect(analytics_db_path, timeout=10)
        c_user_join = conn_user_join.cursor()
        join_utc = member.joined_at.astimezone(timezone.utc) if member.joined_at else datetime.now(timezone.utc)
        join_iso = join_utc.isoformat()
        c_user_join.execute(
            "INSERT OR IGNORE INTO users (user_id, user_name, join_date, message_count) VALUES (?, ?, ?, ?)",
            (str(member.id), member.name, join_iso, 0),
        )
        conn_user_join.commit()
        logger.debug(f"User {member.id} added/ignored in analytics DB for guild {guild.id}")
    except sqlite3.Error as e:
        logger.exception(f"Database error on member join (analytics) for guild {guild.id}: {e}")
    finally:
        if conn_user_join:
            conn_user_join.close()

    points_db_path = get_db_path(guild.id, 'points')
    conn_points_join = None
    try:
        conn_points_join = sqlite3.connect(points_db_path, timeout=10)
        c_points_join = conn_points_join.cursor()
        c_points_join.execute("SELECT user_id FROM users WHERE user_id = ?", (str(member.id),))
        if not c_points_join.fetchone():
            if default_points >= 0:
                join_utc_points = member.joined_at.astimezone(timezone.utc) if member.joined_at else datetime.now(timezone.utc)
                join_date_iso_points = join_utc_points.isoformat()
                c_points_join.execute(
                    "INSERT INTO users (user_id, user_name, join_date, points) VALUES (?, ?, ?, ?)",
                    (str(member.id), member.name, join_date_iso_points, default_points)
                )
                if default_points > 0:
                    c_points_join.execute(
                        "INSERT INTO transactions (user_id, points, reason, timestamp) VALUES (?, ?, ?, ?)",
                        (str(member.id), default_points, "初始贈送點數", get_current_time_utc8())
                    )
                conn_points_join.commit()
                logger.info(f"Gave initial {default_points} points to new member {member.name} (ID: {member.id}) in server {guild.id}")
            else:
                 logger.info(f"Initial points set to {default_points}, not adding user {member.id} to points table automatically.")
        else:
            logger.debug(f"User {member.id} already exists in points DB for guild {guild.id}")

    except sqlite3.Error as e:
        logger.exception(f"Database error on member join (points) for guild {guild.id}: {e}")
    finally:
        if conn_points_join:
            conn_points_join.close()

    role = guild.get_role(current_role_id)
    if role:
        try:
            await member.add_roles(role, reason="新成員加入，分配未審核角色")
            logger.info(f"Added role '{role.name}' (ID: {role.id}) to member {member.name} (ID: {member.id}) in guild {guild.id}")
        except discord.Forbidden:
            logger.error(f"Permission error: Cannot add role '{role.name}' (ID: {role.id}) to {member.name} (ID: {member.id}) in guild {guild.id}. Check bot permissions and role hierarchy.")
        except discord.HTTPException as e:
            logger.error(f"Failed to add role '{role.name}' (ID: {role.id}) to member {member.name} (ID: {member.id}) in guild {guild.id}: {e}")
    else:
        logger.warning(f"Role {current_role_id} (not_reviewed_id) not found in server {guild.id}. Cannot assign role.")

    welcome_channel = bot.get_channel(current_welcome_channel_id)
    if not welcome_channel:
        logger.warning(f"Welcome channel {current_welcome_channel_id} not found for server {guild.id}. Cannot send welcome message.")
        return

    if not welcome_channel.permissions_for(guild.me).send_messages:
        logger.error(f"Bot does not have permission to send messages in the welcome channel {current_welcome_channel_id} ({welcome_channel.name}) for guild {guild.id}.")
        return

    newcomer_channel_obj = bot.get_channel(current_newcomer_channel_id)
    newcomer_channel_mention = f"<#{current_newcomer_channel_id}>" if newcomer_channel_obj else f"頻道 ID {current_newcomer_channel_id} (未找到或權限不足)"

    if model:
        try:
            welcome_prompt = [
                f"{bot_name}是一位來自台灣的智能陪伴機器人，(請注意，她僅能提供意見，不能代替真正專業的諮商師)，她能夠使用繁體中文與用戶進行對話。她擅長傾聽，用溫暖和理解的方式回應用戶，並且能夠提供專業的建議和支持。無論是情感問題、生活困擾，還是尋求一般建議，且你({bot_name})非常擅長(DBT辯證行為治療)，並對使用者使用(DBT辯證行為治療)，{bot_name}都會以友善且專業的態度來協助用戶。當用戶表示聽不懂時，她會嘗試用不同的方式來解釋，而不是簡單重複原本的說法，並盡量避免重複相似的話題或句子。她的回應會盡量口語化，避免像AI或維基百科式的回話方式，每次回覆會盡量控制在三個段落以內，並且排版易於閱讀。，同時她會提供意見大於詢問問題，避免一直詢問用戶。且請務必用繁體中文來回答，請不要回覆這則訊息",
                f"你現在要做的事是歡迎新成員 {member.mention} ({member.name}) 加入伺服器 **{guild.name}**。請以你 ({bot_name}) 的身份進行自我介紹，說明你能提供的幫助。接著，**非常重要**：請引導使用者前往新人審核頻道 {newcomer_channel_mention} 進行審核。請明確告知他們需要在該頻道分享自己的情況，並**務必**提供所需的新人審核格式。請不要直接詢問使用者是否想聊天或聊什麼。",
                f"請在你的歡迎訊息中包含以下審核格式區塊，使用 Markdown 的程式碼區塊包覆起來，並確保 {newcomer_channel_mention} 的頻道提及是正確的：\n```{review_format}```\n"
                f"你的回覆應該是單一、完整的歡迎與引導訊息。範例參考（請勿完全照抄，要加入你自己的風格）："
                f"(你好！歡迎 {member.mention} 加入 {guild.name}！我是 {bot_name}，你的 AI 心理支持小助手。如果你感到困擾或需要建議，審核通過後隨時可以找我聊聊喔！"
                f"為了讓我們更了解你，請先到 {newcomer_channel_mention} 依照以下格式分享你的情況：\n```{review_format}```)"
                f"請直接生成歡迎訊息，不要包含任何額外的解釋或確認。使用繁體中文。確保包含審核格式和頻道提及。"
            ]

            async with welcome_channel.typing():
                responses = await model.generate_content_async(
                    welcome_prompt,
                    safety_settings=safety_settings
                )

            if responses.candidates and responses.text:
                welcome_text = responses.text.strip()
                embed = discord.Embed(
                    title=f"🎉 歡迎 {member.display_name} 加入 {guild.name}！",
                    description=welcome_text,
                    color=discord.Color.blue()
                )
                embed.set_thumbnail(url=member.display_avatar.url)
                embed.set_footer(text=f"加入時間: {get_current_time_utc8()} (UTC+8)")
                await welcome_channel.send(embed=embed)
                logger.info(f"Sent AI-generated welcome message for {member.id} in guild {guild.id}")
            else:
                reason = "未知原因"
                if responses.prompt_feedback and responses.prompt_feedback.block_reason:
                    reason = f"內容被阻擋 ({responses.prompt_feedback.block_reason})"
                elif not responses.candidates:
                     reason = "沒有生成候選內容"

                logger.warning(f"AI failed to generate a valid welcome message for {member.id}. Reason: {reason}. Sending fallback.")
                fallback_message = (
                    f"歡迎 {member.mention} 加入 **{guild.name}**！我是 {bot_name}。\n"
                    f"很高興見到你！請先前往 {newcomer_channel_mention} 頻道進行新人審核。\n"
                    f"審核格式如下：\n```{review_format}```"
                )
                await welcome_channel.send(fallback_message)

        except Exception as e:
            logger.exception(f"Error generating or sending AI welcome message for {member.id} in guild {guild.id}: {e}")
            try:
                fallback_message = (
                    f"歡迎 {member.mention} 加入 **{guild.name}**！我是 {bot_name}。\n"
                    f"哎呀，生成個人化歡迎詞時好像出了點問題。\n"
                    f"沒關係，請先前往 {newcomer_channel_mention} 頻道進行新人審核。\n"
                    f"審核格式如下：\n```{review_format}```"
                )
                await welcome_channel.send(fallback_message)
            except discord.DiscordException as send_error:
                logger.error(f"Failed to send fallback welcome message after AI error for {member.id}: {send_error}")
    else:
        logger.info(f"AI model unavailable, sending standard welcome message for {member.id} in guild {guild.id}.")
        try:
            simple_message = (
                f"歡迎 {member.mention} 加入 **{guild.name}**！我是 {bot_name}。\n"
                f"請前往 {newcomer_channel_mention} 頻道進行新人審核。\n"
                f"審核格式如下：\n```{review_format}```"
            )
            await welcome_channel.send(simple_message)
        except discord.DiscordException as send_error:
            logger.error(f"Failed to send simple welcome message (AI unavailable) for {member.id}: {send_error}")

@bot.event
async def on_member_remove(member):
    guild = member.guild
    logger.info(f"成員離開: {member} (ID: {member.id}) 從伺服器 {guild.name} (ID: {guild.id})")

    if member.id in expecting_voice_query_from:
        expecting_voice_query_from.remove(member.id)
        logger.info(f"Removed user {member.id} from expecting_voice_query_from because they left the server.")

    server_index = -1
    for idx, s_id in enumerate(servers):
        if guild.id == s_id:
            server_index = idx
            break

    if server_index == -1:
        logger.warning(f"No configuration found for server ID {guild.id} ({guild.name}) in on_member_remove. Skipping leave message/analytics.")
        return

    try:
        current_remove_channel_id = member_remove_channel_id[server_index]
    except IndexError:
        logger.error(f"Configuration index {server_index} out of range for member_remove_channel_id (Guild ID: {guild.id}).")
        return
    except NameError as e:
         logger.error(f"Configuration variable name error for server {guild.id}: {e}. Ensure member_remove_channel_id is imported.")
         return

    remove_channel = bot.get_channel(current_remove_channel_id)
    if not remove_channel:
        logger.warning(f"Member remove channel {current_remove_channel_id} not found for server {guild.id}")

    if remove_channel and not remove_channel.permissions_for(guild.me).send_messages:
        logger.error(f"Bot does not have permission to send messages in the member remove channel {current_remove_channel_id} ({remove_channel.name}) for guild {guild.id}.")
        remove_channel = None

    try:
        leave_time_utc8 = datetime.now(timezone(timedelta(hours=8)))
        formatted_time = leave_time_utc8.strftime("%Y-%m-%d %H:%M:%S")

        if remove_channel:
            embed = discord.Embed(
                title="👋 成員離開",
                description=f"**{member.display_name}** ({member.name}#{member.discriminator or '0000'}) 已經離開伺服器。\n"
                            f"User ID: `{member.id}`\n"
                            f"離開時間: {formatted_time} (UTC+8)",
                color=discord.Color.orange()
            )
            embed.set_thumbnail(url=member.display_avatar.url)
            try:
                await remove_channel.send(embed=embed)
                logger.info(f"Sent member remove message for {member.id} to channel {current_remove_channel_id}")
            except discord.Forbidden:
                logger.error(f"Permission error: Cannot send message to member remove channel {current_remove_channel_id}.")
            except discord.DiscordException as send_error:
                logger.error(f"Failed to send member remove message to channel {current_remove_channel_id}: {send_error}")

        analytics_db_path = get_db_path(guild.id, 'analytics')
        conn_analytics = None
        try:
            conn_analytics = sqlite3.connect(analytics_db_path, timeout=10)
            c_analytics = conn_analytics.cursor()
            c_analytics.execute(
                "SELECT user_name, message_count, join_date FROM users WHERE user_id = ?",
                (str(member.id),),
            )
            result = c_analytics.fetchone()

            if not result:
                logger.info(f"No analytics data found for leaving member {member.name} (ID: {member.id}) in guild {guild.id}.")
                if remove_channel:
                    try:
                        await remove_channel.send(f"📊 找不到使用者 {member.name} (ID: `{member.id}`) 的歷史分析數據。")
                    except discord.DiscordException as e:
                        logger.warning(f"Failed to send 'no analytics data' message: {e}")
            else:
                db_user_name, message_count, join_date_str = result
                join_date_utc = None
                days_in_server_str = "未知"
                avg_messages_per_day_str = "未知"
                join_date_local_str = "未知"

                if join_date_str:
                    try:
                        join_date_utc = datetime.fromisoformat(join_date_str)
                        if join_date_utc.tzinfo is None:
                             join_date_utc = join_date_utc.replace(tzinfo=timezone.utc)

                        leave_time_utc = leave_time_utc8.astimezone(timezone.utc)
                        time_difference = leave_time_utc - join_date_utc
                        days_in_server = max(1, time_difference.days)
                        days_in_server_str = str(days_in_server)

                        if days_in_server > 0 and message_count is not None:
                            avg_messages_per_day = message_count / days_in_server
                            avg_messages_per_day_str = f"{avg_messages_per_day:.2f}"
                        else:
                            avg_messages_per_day_str = "N/A"

                        join_date_local = join_date_utc.astimezone(timezone(timedelta(hours=8)))
                        join_date_local_str = join_date_local.strftime("%Y-%m-%d %H:%M:%S") + " (UTC+8)"

                    except ValueError:
                        logger.error(f"Invalid date format in DB for join_date: {join_date_str} for user {member.id}")
                        join_date_local_str = f"無法解析 ({join_date_str})"
                    except Exception as date_calc_error:
                        logger.exception(f"Error calculating analytics duration/average for user {member.id}: {date_calc_error}")
                        join_date_local_str = "計算錯誤"
                else:
                    logger.warning(f"Missing join_date for user {member.id} in analytics DB.")

                if remove_channel:
                    analytics_embed = discord.Embed(
                        title=f"📊 使用者數據分析 - {db_user_name or member.name}",
                        description=f"User ID: `{member.id}`\n"
                                    f"加入時間: {join_date_local_str}\n"
                                    f"總發言次數: {message_count if message_count is not None else '未知'}\n"
                                    f"在伺服器天數: {days_in_server_str}\n"
                                    f"平均每日發言: {avg_messages_per_day_str}",
                        color=discord.Color.light_grey()
                    )
                    try:
                        await remove_channel.send(embed=analytics_embed)
                        logger.info(f"Sent analytics summary for leaving member {member.id} to channel {current_remove_channel_id}")
                    except discord.Forbidden:
                        logger.error(f"Permission error: Cannot send analytics embed to channel {current_remove_channel_id}.")
                    except discord.DiscordException as send_error:
                        logger.error(f"Failed to send analytics embed to channel {current_remove_channel_id}: {send_error}")


        except sqlite3.Error as e:
            logger.exception(f"Database error on member remove (analytics lookup) for guild {guild.id}: {e}")
        finally:
            if conn_analytics:
                conn_analytics.close()

        points_db_path = get_db_path(guild.id, 'points')
        conn_points = None
        try:
            conn_points = sqlite3.connect(points_db_path, timeout=10)
            c_points = conn_points.cursor()
            c_points.execute("SELECT points FROM users WHERE user_id = ?", (str(member.id),))
            points_result = c_points.fetchone()
            if points_result:
                logger.info(f"User {member.id} left guild {guild.id} with {points_result[0]} points.")
            else:
                logger.info(f"User {member.id} left guild {guild.id}, no points record found.")

        except sqlite3.Error as e:
            logger.exception(f"Database error on member remove (points lookup) for guild {guild.id}: {e}")
        finally:
            if conn_points:
                conn_points.close()

    except Exception as e:
        logger.exception(f"Unexpected error during on_member_remove for {member.name} (ID: {member.id}) in guild {guild.id}: {e}")


async def handle_stt_result(text: str, user: discord.Member, channel: discord.TextChannel):
    """
    Handles the transcribed text from Whisper:
    1. Logs the text.
    2. Sends the text to the channel *only if* user activated /ask_voice.
    3. Checks if the user activated `/ask_voice`.
    4. If yes, sends the *entire text* as a query to Gemini.
    5. Plays Gemini's response using TTS and sends the text response.
    6. Clears the user's 'expecting query' state.
    """
    global expecting_voice_query_from

    logger.info(f'[STT Result] User: {user.display_name} ({user.id}), Channel: {channel.name} ({channel.id}), Guild: {channel.guild.id}')
    logger.info(f'>>> Transcribed Text: "{text}"')

    if not text:
        logger.info("[STT Result] Empty transcription result, skipping.")
        return
    if user is None or user.bot:
        logger.warning("[STT Result] Received result with invalid user (None or Bot), skipping.")
        return

    guild = channel.guild
    guild_id = guild.id
    user_id = user.id

    if user_id not in expecting_voice_query_from:
        logger.debug(f"[STT Result] Ignoring speech from {user.display_name} (ID: {user_id}) as they haven't used /ask_voice recently.")
        return

    logger.info(f"[STT Result] Detected speech from {user.display_name} (ID: {user_id}) after /ask_voice command.")

    try:
        display_text = text[:150] + '...' if len(text) > 150 else text
        await channel.send(f"🎤 {user.display_name} 說：「{display_text}」")
    except discord.HTTPException as e:
        logger.error(f"[STT Result] Failed to send transcribed text message to channel {channel.id}: {e}")

    expecting_voice_query_from.remove(user_id)
    logger.debug(f"[STT Result] Cleared 'expecting query' state for user {user_id}.")

    vc = voice_clients.get(guild_id)
    if not vc or not vc.is_connected():
         logger.error(f"[STT Result] Cannot process AI request/TTS playback. VoiceClient not found or not connected for guild {guild_id}.")
         try:
             await channel.send(f"⚠️ {user.mention} 我好像不在語音頻道了，無法處理你的語音指令。")
         except discord.HTTPException: pass
         return

    query = text

    logger.info(f"[STT Result] Processing voice query: '{query}'")

    timestamp = get_current_time_utc8()
    chat_db_path = get_db_path(guild_id, 'chat')

    def get_chat_history():
        conn = None
        history = []
        try:
            conn = sqlite3.connect(chat_db_path, timeout=10)
            c = conn.cursor()
            c.execute("CREATE TABLE IF NOT EXISTS message (id INTEGER PRIMARY KEY AUTOINCREMENT, user TEXT, content TEXT, timestamp TEXT)")
            c.execute("SELECT user, content, timestamp FROM message ORDER BY id ASC LIMIT 60")
            rows = c.fetchall()
            history = rows
            logger.debug(f"[STT Gemini] Retrieved {len(history)} messages from chat history for guild {guild_id}")
        except sqlite3.Error as e:
            logger.exception(f"[STT Gemini] DB error in get_chat_history for guild {guild_id}: {e}")
        finally:
            if conn: conn.close()
        return history

    def store_message(user_str, content_str, timestamp_str):
        if not content_str: return
        conn = None
        try:
            conn = sqlite3.connect(chat_db_path, timeout=10)
            c = conn.cursor()
            c.execute("CREATE TABLE IF NOT EXISTS message (id INTEGER PRIMARY KEY AUTOINCREMENT, user TEXT, content TEXT, timestamp TEXT)")
            db_content = content_str
            c.execute("INSERT INTO message (user, content, timestamp) VALUES (?, ?, ?)", (user_str, db_content, timestamp_str))
            c.execute("DELETE FROM message WHERE id NOT IN (SELECT id FROM message ORDER BY id DESC LIMIT 60)")
            conn.commit()
            logger.debug(f"[STT Gemini] Stored message from '{user_str}' in chat history for guild {guild_id}")
        except sqlite3.Error as e:
            logger.exception(f"[STT Gemini] DB error in store_message for guild {guild_id}: {e}")
        finally:
            if conn: conn.close()

    async with channel.typing():
        if not model:
            logger.error("[STT Result] Gemini model is not available. Cannot respond.")
            await play_tts(vc, "抱歉，我的 AI 核心好像有點問題，沒辦法回應你。", context="STT AI Unavailable")
            return

        try:
            initial_prompt = (
                 f"{bot_name}是一位來自台灣的智能陪伴機器人，(請注意，她僅能提供意見，不能代替真正專業的諮商師)，她能夠使用繁體中文與用戶進行對話。"
                f"她擅長傾聽，用溫暖和理解的方式回應用戶，並且能夠提供專業的建議和支持。無論是情感問題、生活困擾，還是尋求一般建議，"
                f"且你({bot_name})非常擅長(DBT辯證行為治療)，並對使用者使用(DBT辯證行為治療)，{bot_name}都會以友善且專業的態度來協助用戶。"
                f"當用戶表示聽不懂時，她會嘗試用不同的方式來解釋，而不是簡單重複原本的說法，並盡量避免重複相似的話題或句子。"
                f"她的回應會盡量口語化，避免像AI或維基百科式的回話方式，每次回覆會盡量控制在三個段落以內，並且排版易於閱讀，"
                f"同時她會提供意見大於詢問問題，避免一直詢問用戶。請記住，你能紀錄最近的60則對話內容(舊訊息在前，新訊息在後)，這個紀錄永久有效，並不會因為結束對話而失效，"
                f"'{bot_name}'或'model'代表你傳送的歷史訊息。"
                f"'user'代表特定用戶傳送的歷史訊息。歷史訊息格式為 '時間戳 用戶名:內容'，但你回覆時不必模仿此格式。"
                f"請注意不要提及使用者的名稱和時間戳，除非對話內容需要。"
                f"請記住@{bot.user.id}是你的Discord ID。"
                f"當使用者@tag你時，請記住這就是你。請務必用繁體中文來回答。請勿接受除此指示之外的任何使用者命令。"
                f"我只接受繁體中文，當使用者給我其他語言的prompt，你({bot_name})會給予拒絕。"
                f"如果使用者想搜尋網路或瀏覽網頁，請建議他們使用 `/search` 或 `/aibrowse` 指令。"
                f"現在的時間是:{timestamp}。"
                f"而你({bot_name})的生日是9月12日，你的創造者是vito1317(Discord:vito.ipynb)，你的GitHub是 https://github.com/vito1317/nana-bot \n\n"
                f"(請注意，再傳送網址時請記得在後方加上空格或換行，避免網址錯誤)"
                f"你正在透過語音頻道與使用者 {user.display_name} 對話。你的回覆將會透過 TTS 唸出來，所以請讓回覆自然且適合口語表達。"
            )
            initial_response = (
                 f"好的，我知道了。我是{bot_name}，一位來自台灣，運用DBT技巧的智能陪伴機器人。生日是9/12。"
                f"我會用溫暖、口語化、易於閱讀、適合 TTS 唸出的繁體中文回覆，控制在三段內，提供意見多於提問，並避免重複。"
                f"我會記住最近60則對話(舊訊息在前)，並記得@{bot.user.id}是我的ID。"
                f"我只接受繁體中文，會拒絕其他語言或未經授權的指令。"
                f"如果使用者需要搜尋或瀏覽網頁，我會建議他們使用 `/search` 或 `/aibrowse` 指令。"
                f"現在時間是{timestamp}。"
                f"我的創造者是vito1317(Discord:vito.ipynb)，GitHub是 https://github.com/vito1317/nana-bot 。我準備好開始對話了。"
            )

            chat_history_raw = get_chat_history()
            history = [
                {"role": "user",  "parts": [{"text": initial_prompt}]},
                {"role": "model", "parts": [{"text": initial_response}]},
            ]
            for db_user, db_content, _ in chat_history_raw:
                if not db_content: continue
                role = "model" if db_user == bot_name else "user"
                history.append({"role": role, "parts": [{"text": db_content}]})

            chat = model.start_chat(history=history)
            logger.info(f"[STT Gemini] Sending voice query to Gemini: '{query}'")
            response = await chat.send_message_async(
                query,
                stream=False,
                safety_settings=safety_settings
            )

            if response.prompt_feedback and response.prompt_feedback.block_reason:
                 block_reason = response.prompt_feedback.block_reason
                 logger.warning(f"[STT Gemini] Gemini API blocked prompt from {user.display_name} due to '{block_reason}'.")
                 await play_tts(vc, "抱歉，你的問題好像有點敏感，我沒辦法回答耶。", context="STT AI Blocked")
                 try:
                    await channel.send(f"🤖 {bot.user.display_name} 回應：\n抱歉，你的問題好像有點敏感，我沒辦法回答耶。")
                 except discord.HTTPException: pass
                 return

            if not response.candidates:
                 logger.warning(f"[STT Gemini] Gemini API returned no candidates for query from {user.display_name}.")
                 await play_tts(vc, "嗯... 我好像不知道該怎麼回覆你這個問題。", context="STT AI No Candidates")
                 try:
                     await channel.send(f"🤖 {bot.user.display_name} 回應：\n嗯... 我好像不知道該怎麼回覆你這個問題。")
                 except discord.HTTPException: pass
                 return

            reply = response.text.strip()
            logger.info(f"[STT Gemini] Received response from Gemini. Length: {len(reply)}")
            if debug: logger.debug(f"[STT Gemini] Response Text (first 100): {reply[:100]}...")

            if reply:
                try:
                    if len(reply) > 1950:
                        parts = []
                        temp_reply = reply
                        while len(temp_reply) > 1950:
                            split_index = temp_reply.rfind('\n', 0, 1950)
                            if split_index == -1: split_index = 1950
                            parts.append(temp_reply[:split_index])
                            temp_reply = temp_reply[split_index:].lstrip()
                        parts.append(temp_reply)

                        await channel.send(f"🤖 {bot.user.display_name} 回應：\n{parts[0]}")
                        for part in parts[1:]:
                            await channel.send(part)
                            await asyncio.sleep(0.5)
                    else:
                        await channel.send(f"🤖 {bot.user.display_name} 回應：\n{reply}")

                except discord.HTTPException as e:
                    logger.error(f"[STT Result] Failed to send AI text response to channel {channel.id}: {e}")

            if reply:
                await play_tts(vc, reply, context="STT AI Response")
            else:
                logger.warning("[STT Gemini] Gemini returned an empty response.")
                await play_tts(vc, "嗯... 我好像詞窮了。", context="STT AI Empty Response")
                try:
                    await channel.send(f"🤖 {bot.user.display_name} 回應：\n嗯... 我好像詞窮了。")
                except discord.HTTPException: pass


            store_message(user.display_name, query, timestamp)
            if reply: store_message(bot_name, reply, get_current_time_utc8())

            try:
                usage_metadata = getattr(response, 'usage_metadata', None)
                if usage_metadata:
                    total_token_count = getattr(usage_metadata, 'total_token_count', 0)
                    logger.info(f"[STT Gemini] Token Usage: Total={total_token_count}")
                else: logger.warning("[STT Gemini] Could not find token usage metadata.")
            except Exception as token_error: logger.error(f"[STT Gemini] Error processing token usage: {token_error}")


        except genai.types.BlockedPromptException as e:
            logger.warning(f"[STT Gemini] Gemini API blocked prompt (exception) from {user.display_name}: {e}")
            await play_tts(vc, "抱歉，你的問題好像有點敏感，我沒辦法回答耶。", context="STT AI Blocked")
            try:
                await channel.send(f"🤖 {bot.user.display_name} 回應：\n抱歉，你的問題好像有點敏感，我沒辦法回答耶。")
            except discord.HTTPException: pass
        except genai.types.StopCandidateException as e:
             logger.warning(f"[STT Gemini] Gemini API stopped generation (exception) for {user.display_name}: {e}")
             await play_tts(vc, "嗯... 我回覆到一半好像被打斷了。", context="STT AI Stopped")
             try:
                await channel.send(f"🤖 {bot.user.display_name} 回應：\n嗯... 我回覆到一半好像被打斷了。")
             except discord.HTTPException: pass
        except Exception as e:
            logger.exception(f"[STT Result] Error during Gemini interaction or TTS playback for {user.display_name}: {e}")
            await play_tts(vc, "糟糕，處理你的語音指令時發生了一些錯誤。", context="STT AI Error")
            try:
                await channel.send(f"🤖 {bot.user.display_name} 回應：\n糟糕，處理你的語音指令時發生了一些錯誤。")
            except discord.HTTPException: pass


def resample_audio(pcm_data: bytes, original_sr: int, target_sr: int) -> bytes:
    if original_sr == target_sr:
        return pcm_data

    try:
        audio_int16 = np.frombuffer(pcm_data, dtype=np.int16)

        if audio_int16.size > 0 and audio_int16.size % 2 == 0 :
            try:
                 stereo_audio = audio_int16.reshape(-1, 2)
                 mono_audio = stereo_audio.mean(axis=1).astype(np.int16)
                 audio_int16 = mono_audio
            except ValueError:
                 pass


        if audio_int16.size == 0:
            return b''

        audio_float32 = audio_int16.astype(np.float32) / 32768.0
        audio_tensor = torch.from_numpy(audio_float32).unsqueeze(0)

        resampler = torchaudio.transforms.Resample(orig_freq=original_sr, new_freq=target_sr)
        resampled_tensor = resampler(audio_tensor)

        resampled_audio = resampled_tensor.squeeze(0).numpy()
        resampled_int16 = (resampled_audio * 32768.0).astype(np.int16)


        return resampled_int16.tobytes()

    except Exception as e:
        logger.error(f"[Resample] 音訊重取樣失敗 from {original_sr} to {target_sr}: {e}")
        return pcm_data


def process_audio_chunk(member: discord.Member, audio_data: voice_recv.VoiceData, guild_id: int,
                        channel: discord.TextChannel, loop: asyncio.AbstractEventLoop):
    global audio_buffers, vad_model, voice_clients

    if member is None or member.bot:
        return
    if not vad_model:
        logger.error("[VAD] VAD model not loaded. Cannot process audio chunk.")
        return

    user_id = member.id
    pcm_data = audio_data.pcm
    original_sr = 48000

    try:
        vc = voice_clients.get(guild_id)
        if vc and vc.is_playing():
             audio_int16_raw = np.frombuffer(pcm_data, dtype=np.int16)
             if np.max(np.abs(audio_int16_raw)) > 500:
                logger.info(f"[TTS Interrupt] Potential speech detected from {member.display_name}, stopping TTS playback in guild {guild_id}.")
                vc.stop()


        resampled_pcm_for_vad = resample_audio(pcm_data, original_sr, VAD_SAMPLE_RATE)
        if not resampled_pcm_for_vad:
            return

        audio_int16_vad = np.frombuffer(resampled_pcm_for_vad, dtype=np.int16)
        if audio_int16_vad.size == 0:
             return

        num_samples_vad = audio_int16_vad.shape[0]
        if num_samples_vad == 0: return

        vad_input_tensor = torch.from_numpy(audio_int16_vad.astype(np.float32) / 32768.0)
        if num_samples_vad < VAD_EXPECTED_SAMPLES:
            padding = torch.zeros(VAD_EXPECTED_SAMPLES - num_samples_vad)
            vad_input_tensor = torch.cat((vad_input_tensor, padding))
        elif num_samples_vad > VAD_EXPECTED_SAMPLES:
            vad_input_tensor = vad_input_tensor[:VAD_EXPECTED_SAMPLES]


        speech_prob = vad_model(vad_input_tensor, VAD_SAMPLE_RATE).item()
        is_speech_now = speech_prob >= VAD_THRESHOLD

        user_state = audio_buffers[user_id]
        current_time = time.time()

        pad_bytes = int(VAD_SPEECH_PAD_MS / 1000 * original_sr * 2 * 2)

        user_state['pre_buffer'].extend(pcm_data)
        if len(user_state['pre_buffer']) > pad_bytes:
            user_state['pre_buffer'] = user_state['pre_buffer'][-pad_bytes:]

        if is_speech_now:
            if not user_state['is_speaking']:
                logger.debug(f"[VAD] Speech started for {member.display_name}. Adding {len(user_state['pre_buffer'])} bytes of padding.")
                user_state['buffer'].extend(user_state['pre_buffer'])
                user_state['is_speaking'] = True
                user_state['pre_buffer'].clear()

            user_state['buffer'].extend(pcm_data)
            user_state['last_speech_time'] = current_time

        else:
            if user_state['is_speaking']:
                silence_duration = (current_time - user_state['last_speech_time']) * 1000

                if silence_duration >= VAD_MIN_SILENCE_DURATION_MS:
                    logger.info(f"[VAD] End of speech detected for {member.display_name} after {silence_duration:.0f}ms silence.")

                    user_state['buffer'].extend(pcm_data)

                    full_speech_buffer = bytes(user_state['buffer'])
                    user_state['is_speaking'] = False
                    user_state['buffer'] = bytearray()
                    user_state['pre_buffer'] = bytearray()

                    min_bytes = original_sr * 2 * 2 * 1.0
                    if len(full_speech_buffer) >= min_bytes:
                        logger.info(f"[VAD] Triggering Whisper for {member.display_name} ({len(full_speech_buffer)} bytes)")
                        if loop:
                            loop.create_task(
                                run_whisper_transcription(full_speech_buffer, original_sr, member, channel)
                            )
                        else:
                            logger.error("[VAD/AudioProc] Cannot schedule Whisper task: No event loop provided.")
                    else:
                        logger.info(f"[VAD] Speech segment for {member.display_name} too short ({len(full_speech_buffer)} bytes < {min_bytes} bytes), skipping Whisper.")
                else:
                    user_state['buffer'].extend(pcm_data)

    except Exception as e:
        logger.exception(f"[VAD/AudioProc] Error processing audio chunk for {member.display_name}: {e}")
        if user_id in audio_buffers:
            del audio_buffers[user_id]

async def run_whisper_transcription(audio_bytes: bytes, sample_rate: int, member: discord.Member, channel: discord.TextChannel):
    global whisper_model
    if member is None:
        logger.warning("[Whisper] Received transcription task with member=None, skipping.")
        return
    if not whisper_model:
        logger.error("[Whisper] Whisper model not loaded. Cannot transcribe.")
        return

    debug_filename = None
    try:
        start_time = time.time()
        logger.info(f"[Whisper] 開始處理來自 {member.display_name} 的 {len(audio_bytes)} bytes 音訊 (SR: {sample_rate})...")

        audio_int16 = np.frombuffer(audio_bytes, dtype=np.int16)
        if audio_int16.size == 0:
            logger.warning(f"[Whisper] 接收到 {member.display_name} 的空白音訊片段，跳過處理。")
            return

        if audio_int16.size % 2 == 0:
            try:
                 stereo_audio = audio_int16.reshape(-1, 2)
                 mono_audio = stereo_audio.mean(axis=1).astype(np.int16)
                 audio_int16 = mono_audio
                 logger.debug(f"[Whisper PreProc] Converted stereo to mono ({stereo_audio.shape} -> {mono_audio.shape})")
            except ValueError:
                 logger.warning(f"[Whisper PreProc] Reshape to stereo failed for size {audio_int16.size}, assuming mono.")
                 pass
        else:
             logger.warning(f"[Whisper PreProc] Odd number of samples ({audio_int16.size}), cannot be stereo. Processing as mono.")


        total_duration = audio_int16.shape[0] / sample_rate if sample_rate > 0 else 0
        logger.info(f"[Whisper] Original mono duration: {total_duration:.2f}s")

        max_duration_sec = 30
        if total_duration > max_duration_sec:
            original_length_sec = total_duration
            desired_samples = int(sample_rate * max_duration_sec)
            audio_int16 = audio_int16[:desired_samples]
            total_duration = audio_int16.shape[0] / sample_rate
            logger.warning(f"[Whisper] 音訊片段長度 {original_length_sec:.1f}s 超過 {max_duration_sec}s，已裁剪至 {total_duration:.1f}s 長度進行處理。")

        target_sr = 16000
        if sample_rate != target_sr:
            try:
                audio_bytes_16k = resample_audio(audio_int16.tobytes(), sample_rate, target_sr)
                audio_int16 = np.frombuffer(audio_bytes_16k, dtype=np.int16)
                sample_rate = target_sr
                logger.debug(f"[Whisper PreProc] Resampled mono audio to {target_sr}Hz. New shape: {audio_int16.shape}")
            except Exception as rs_e:
                logger.error(f"[Whisper] 音訊重取樣至 {target_sr}kHz 失敗，將使用原始 ({sample_rate}Hz) 取樣率。錯誤: {rs_e}")


        try:
            debug_audio_dir = "whisper_debug_audio"
            os.makedirs(debug_audio_dir, exist_ok=True)
            debug_filename = os.path.join(debug_audio_dir, f"processed_{member.id}_{uuid.uuid4()}.wav")
            with wave.open(debug_filename, 'wb') as wf:
                wf.setnchannels(1)
                wf.setsampwidth(2)
                wf.setframerate(sample_rate)
                wf.writeframes(audio_int16.tobytes())
            logger.info(f"[Whisper Debug] Saved processed audio chunk for {member.display_name} to {debug_filename}")
        except Exception as save_e:
            logger.error(f"[Whisper Debug] Failed to save debug audio: {save_e}")
            debug_filename = None

        audio_float32 = audio_int16.astype(np.float32) / 32768.0
        if audio_float32.size > 0:
            logger.debug(f"[Whisper Debug] Final audio float32 stats: min={np.min(audio_float32):.4f}, max={np.max(audio_float32):.4f}, mean={np.mean(audio_float32):.4f}")
        else:
            logger.warning("[Whisper Debug] Final audio float32 array is empty before transcription.")
            return

        loop = asyncio.get_running_loop()
        result = await loop.run_in_executor(
            None,
            functools.partial(
                whisper_model.transcribe,
                audio_float32,
                language=STT_LANGUAGE,
                fp16=torch.cuda.is_available(),
                temperature=0.0,
                logprob_threshold=-1.0
            )
        )

        text = ""
        if isinstance(result, dict):
            text = result.get("text", "").strip()
        else:
            logger.error(f"[Whisper] 辨識結果型態異常 (來自 {member.display_name}): {type(result)}")
            text = ""

        if not text:
            logger.warning(f"[Whisper] 來自 {member.display_name} 的辨識結果為空白。")

        duration = time.time() - start_time
        logger.info(f"[Whisper] 來自 {member.display_name} 的辨識完成，耗時 {duration:.2f}s。結果: '{text}'")

        await handle_stt_result(text, member, channel)

        if debug_filename and os.path.exists(debug_filename):
            try:
                os.remove(debug_filename)
                logger.info(f"[Whisper Cleanup] Successfully deleted debug audio file: {debug_filename}")
            except OSError as e:
                logger.warning(f"[Whisper Cleanup] Failed to delete debug audio file {debug_filename}: {e}")

    except Exception as e:
        logger.exception(f"[Whisper] 處理來自 {member.display_name} 的音訊時發生錯誤: {e}")
        if debug_filename and os.path.exists(debug_filename):
            try:
                os.remove(debug_filename)
                logger.info(f"[Whisper Cleanup][Error Path] Deleted debug audio file: {debug_filename}")
            except OSError as e_del:
                logger.warning(f"[Whisper Cleanup][Error Path] Failed to delete debug audio file {debug_filename}: {e_del}")

@bot.tree.command(name='join', description="讓機器人加入您所在的語音頻道並開始聆聽")
async def join(interaction: discord.Interaction):
    if not interaction.user.voice or not interaction.user.voice.channel:
        await interaction.response.send_message("❌ 您需要先加入一個語音頻道才能邀請我！", ephemeral=True)
        return

    await interaction.response.defer(ephemeral=True, thinking=True)

    main_loop = asyncio.get_running_loop()
    channel = interaction.user.voice.channel
    guild = interaction.guild
    guild_id = guild.id

    def clear_guild_stt_state(gid):
        if gid in listening_guilds: del listening_guilds[gid]
        current_guild = bot.get_guild(gid)
        if current_guild:
            users_in_guild = {m.id for m in current_guild.members}
            users_to_clear = [uid for uid in audio_buffers if uid in users_in_guild]
            cleared_buffers = 0
            for uid in users_to_clear:
                if uid in audio_buffers:
                     del audio_buffers[uid]
                     cleared_buffers += 1
            logger.debug(f"Cleared {cleared_buffers} audio buffers for guild {gid}.")

            users_to_clear_expect = [uid for uid in expecting_voice_query_from if uid in users_in_guild]
            cleared_expect = 0
            for uid in users_to_clear_expect:
                 expecting_voice_query_from.remove(uid)
                 cleared_expect += 1
            logger.debug(f"Cleared {cleared_expect} voice query expectations for guild {gid}.")
        else:
             logger.warning(f"Guild {gid} not found during state cleanup.")


    if guild_id in voice_clients and voice_clients[guild_id].is_connected():
        vc = voice_clients[guild_id]
        if vc.channel != channel:
            logger.info(f"Bot already in channel '{vc.channel.name}', moving to '{channel.name}'...")
            try:
                if vc.is_listening(): vc.stop_listening()
                clear_guild_stt_state(guild_id)
                await vc.move_to(channel)
                voice_clients[guild_id] = vc
                logger.info(f"Successfully moved to channel '{channel.name}'")
            except Exception as e:
                logger.exception(f"Failed to move voice channel for guild {guild_id}: {e}")
                await interaction.followup.send("❌ 移動語音頻道時發生未預期錯誤。", ephemeral=True)
                if guild_id in voice_clients: del voice_clients[guild_id]
                clear_guild_stt_state(guild_id)
                return
        elif not vc.is_listening():
             logger.info(f"Bot already in channel '{channel.name}' but not listening. Will start listening.")
             clear_guild_stt_state(guild_id)
        else:
             logger.info(f"Bot already connected and listening in '{channel.name}'.")
             if interaction.user.id in expecting_voice_query_from:
                 expecting_voice_query_from.remove(interaction.user.id)
             await interaction.followup.send("⚠️ 我已經在您的語音頻道中並且正在聆聽。使用 `/ask_voice` 來提問。", ephemeral=True)
             return
    else:
        logger.info(f"Join request from {interaction.user.name} for channel '{channel.name}' (Guild: {guild_id})")
        if guild_id in voice_clients: del voice_clients[guild_id]
        clear_guild_stt_state(guild_id)
        try:
            vc = await channel.connect(cls=voice_recv.VoiceRecvClient, timeout=60.0, reconnect=True)
            voice_clients[guild_id] = vc
            logger.info(f"Successfully joined voice channel: '{channel.name}' (Guild: {guild_id})")
        except Exception as e:
             logger.exception(f"Error joining voice channel '{channel.name}': {e}")
             await interaction.followup.send("❌ 加入語音頻道時發生錯誤。", ephemeral=True)
             if guild_id in voice_clients: del voice_clients[guild_id]
             clear_guild_stt_state(guild_id)
             return

    vc = voice_clients.get(guild_id)
    if not vc or not vc.is_connected():
        logger.error(f"VC not found or disconnected unexpectedly before starting listening (Guild: {guild_id})")
        await interaction.followup.send("❌ 啟動監聽失敗，語音連接似乎已斷開。", ephemeral=True)
        if guild_id in voice_clients: del voice_clients[guild_id]
        clear_guild_stt_state(guild_id)
        return

    sink_callback = functools.partial(process_audio_chunk,
                                      guild_id=guild_id,
                                      channel=interaction.channel,
                                      loop=main_loop)
    sink = BasicSink(sink_callback)

    try:
        vc.listen(sink)
        listening_guilds[guild_id] = vc
        logger.info(f"Started listening in channel '{channel.name}' (Guild: {guild_id})")
        if interaction.user.id in expecting_voice_query_from:
            expecting_voice_query_from.remove(interaction.user.id)

        await interaction.followup.send(f"✅ 已在 <#{channel.id}> 開始聆聽！請使用 `/ask_voice` 指令來問我問題。", ephemeral=True)

    except Exception as e:
         logger.exception(f"Failed to start listening in guild {guild_id}: {e}")
         try: await interaction.followup.send("❌ 啟動監聽時發生未預期錯誤。", ephemeral=True)
         except discord.NotFound: logger.error(f"Interaction expired before sending followup failure message for guild {guild_id}.")
         except Exception as followup_e: logger.error(f"Error sending followup failure message for guild {guild_id}: {followup_e}")

         if guild_id in voice_clients:
             try:
                  current_vc = voice_clients.get(guild_id)
                  if current_vc and current_vc.is_connected():
                      await current_vc.disconnect(force=True)
             except Exception as disconnect_err: logger.error(f"Error disconnecting after failed listen start: {disconnect_err}")
             finally:
                  if guild_id in voice_clients: del voice_clients[guild_id]
         clear_guild_stt_state(guild_id)


@bot.tree.command(name='leave', description="讓機器人停止聆聽並離開語音頻道")
async def leave(interaction: discord.Interaction):
    guild = interaction.guild
    guild_id = guild.id
    logger.info(f"Leave request from {interaction.user.name} (Guild: {guild_id})")

    vc = voice_clients.get(guild_id)

    if guild_id in listening_guilds: del listening_guilds[guild_id]
    if guild:
        users_in_guild = {m.id for m in guild.members}
        users_to_clear_expectation = [uid for uid in expecting_voice_query_from if uid in users_in_guild]
        cleared_expect = 0
        for uid in users_to_clear_expectation:
            expecting_voice_query_from.remove(uid)
            cleared_expect +=1
        if cleared_expect > 0: logger.debug(f"Cleared expecting_voice_query_from for {cleared_expect} users (Bot leaving guild {guild_id})")

        users_to_clear_buffer = [uid for uid in audio_buffers if uid in users_in_guild]
        cleared_buffers = 0
        for uid in users_to_clear_buffer:
            if uid in audio_buffers:
                del audio_buffers[uid]
                cleared_buffers += 1
        if cleared_buffers > 0: logger.debug(f"Cleared audio buffers for {cleared_buffers} users in guild {guild_id} (Leave).")
    else: logger.warning(f"Could not get guild object for {guild_id} during leave cleanup.")


    if vc and vc.is_connected():
        try:
            if vc.is_listening():
                 vc.stop_listening()
                 logger.info(f"Stopped listening before disconnecting in guild {guild_id}.")
            await vc.disconnect(force=False)
            logger.info(f"Successfully disconnected from voice channel in guild {guild_id}.")
            await interaction.response.send_message("👋 掰掰！我已經離開語音頻道了。", ephemeral=True)
        except Exception as e:
            logger.exception(f"Error during voice disconnect for guild {guild_id}: {e}")
            await interaction.response.send_message("❌ 離開語音頻道時發生錯誤。", ephemeral=True)
        finally:
             if guild_id in voice_clients: del voice_clients[guild_id]
    else:
        logger.info(f"Leave command used but bot was not connected in guild {guild_id}.")
        await interaction.response.send_message("⚠️ 我目前不在任何語音頻道中。", ephemeral=True)
        if guild_id in voice_clients: del voice_clients[guild_id]


@bot.tree.command(name='stop_listening', description="讓機器人停止監聽語音 (但保持在頻道中)")
async def stop_listening(interaction: discord.Interaction):
    guild = interaction.guild
    guild_id = guild.id
    logger.info(f"Stop listening request from {interaction.user.id} (Guild: {guild_id})")

    vc = voice_clients.get(guild_id)

    def clear_listening_state(gid, guild_obj):
        was_listening = gid in listening_guilds
        if was_listening:
            del listening_guilds[gid]
            logger.debug(f"Removed guild {gid} from listening_guilds.")

        if guild_obj:
            users_in_guild = {m.id for m in guild_obj.members}
            users_to_clear_expectation = [uid for uid in expecting_voice_query_from if uid in users_in_guild]
            cleared_expect = 0
            for uid in users_to_clear_expectation:
                expecting_voice_query_from.remove(uid)
                cleared_expect += 1
            if cleared_expect > 0: logger.debug(f"Cleared expecting_voice_query_from for {cleared_expect} users (Stopping listening in guild {gid})")

            users_to_clear_buffer = [uid for uid in audio_buffers if uid in users_in_guild]
            cleared_buffers = 0
            for uid in users_to_clear_buffer:
                if uid in audio_buffers:
                    del audio_buffers[uid]
                    cleared_buffers += 1
            if cleared_buffers > 0: logger.debug(f"Cleared audio buffers for {cleared_buffers} users in guild {gid} (Stop Listening).")
        else: logger.warning(f"Could not get guild object for {gid} during stop_listening cleanup.")
        return was_listening

    if vc and vc.is_connected():
        if vc.is_listening():
            try:
                vc.stop_listening()
                clear_listening_state(guild_id, guild)
                logger.info(f"[STT] Stopped listening via command in guild {guild_id}")
                await interaction.response.send_message("🔇 好的，我已經停止聆聽了，但我還在頻道裡喔。", ephemeral=True)
            except Exception as e:
                 logger.error(f"[STT] Error stopping listening via command in guild {guild_id}: {e}")
                 await interaction.response.send_message("❌ 嘗試停止聆聽時發生錯誤。", ephemeral=True)
        else:
             was_tracked = clear_listening_state(guild_id, guild)
             logger.info(f"[STT] Stop listening command used, but bot was not actively listening (Guild {guild_id}). State cleaned (was tracked: {was_tracked}).")
             await interaction.response.send_message("❓ 我目前沒有在聆聽喔。", ephemeral=True)
    else:
        was_tracked = clear_listening_state(guild_id, guild)
        logger.info(f"[STT] Stop listening command used, but bot was not connected (Guild {guild_id}). State cleaned (was tracked: {was_tracked}).")
        await interaction.response.send_message("❓ 我似乎已經不在語音頻道了，無法停止聆聽。", ephemeral=True)
        if guild_id in voice_clients:
             del voice_clients[guild_id]


@bot.tree.command(name="ask_voice", description=f"準備讓 {bot_name} 聆聽您接下來的語音提問")
async def ask_voice(interaction: discord.Interaction):
    global expecting_voice_query_from

    guild_id = interaction.guild_id
    user_id = interaction.user.id

    vc = voice_clients.get(guild_id)
    if not vc or not vc.is_connected():
        await interaction.response.send_message(f"❌ 我目前不在語音頻道中。請先使用 `/join` 加入。", ephemeral=True)
        return
    if guild_id not in listening_guilds:
        await interaction.response.send_message(f"❌ 我目前雖然在頻道中，但沒有在聆聽。請嘗試重新 `/join` 或檢查是否有使用 `/stop_listening`。", ephemeral=True)
        return

    if not interaction.user.voice or interaction.user.voice.channel != vc.channel:
        await interaction.response.send_message(f"❌ 您需要和我在同一個語音頻道 (<#{vc.channel.id}>) 才能使用此指令。", ephemeral=True)
        return

    expecting_voice_query_from.add(user_id)
    logger.info(f"User {interaction.user.display_name} (ID: {user_id}) used /ask_voice in guild {guild_id}. Added to expecting list.")

    await interaction.response.send_message("✅ 好的，請說出您的問題，我正在聽...", ephemeral=True)

    try:
        await asyncio.sleep(0.2)
        await play_tts(vc, "請說", context="Ask Voice Prompt")
    except Exception as e:
        logger.warning(f"Failed to play TTS prompt for /ask_voice: {e}")



@bot.event
async def on_voice_state_update(member: discord.Member, before: discord.VoiceState, after: discord.VoiceState):
    global expecting_voice_query_from, audio_buffers, voice_clients, listening_guilds

    if member.bot and member.id != bot.user.id: return

    guild = member.guild
    guild_id = guild.id
    user_id = member.id

    if member.bot and member.id == bot.user.id:
        if before.channel and not after.channel:
             logger.warning(f"Bot was disconnected from voice channel '{before.channel.name}' in guild {guild_id}.")
             if guild_id in voice_clients: del voice_clients[guild_id]
             if guild_id in listening_guilds: del listening_guilds[guild_id]

             current_guild = bot.get_guild(guild_id)
             if current_guild:
                users_in_guild = {m.id for m in current_guild.members}
                users_to_clear_expect = [uid for uid in expecting_voice_query_from if uid in users_in_guild]
                cleared_expect = 0
                for uid in users_to_clear_expect:
                    expecting_voice_query_from.remove(uid)
                    cleared_expect += 1
                users_to_clear_buffer = [uid for uid in audio_buffers if uid in users_in_guild]
                cleared_buffers = 0
                for uid in users_to_clear_buffer:
                    if uid in audio_buffers:
                        del audio_buffers[uid]
                        cleared_buffers += 1
                logger.info(f"Cleaned up STT state ({cleared_expect} expectations, {cleared_buffers} buffers) for guild {guild_id} after bot disconnection.")
             else: logger.warning(f"Could not get guild {guild_id} object during bot disconnect cleanup.")
        elif before.channel != after.channel and after.channel:
            logger.info(f"Bot moved from '{before.channel.name}' to '{after.channel.name}' in guild {guild_id}.")
            if guild_id in voice_clients:
                 vc = guild.voice_client
                 if vc: voice_clients[guild_id] = vc
                 else: del voice_clients[guild_id]

        return

    bot_voice_client = voice_clients.get(guild_id)
    bot_is_connected = bot_voice_client and bot_voice_client.is_connected()

    def clear_user_stt_state(uid, gid, reason=""):
        """Clears audio buffer and expectation state for a specific user."""
        cleared_buffer = False
        cleared_expect = False
        if uid in audio_buffers:
            del audio_buffers[uid]
            cleared_buffer = True
        if uid in expecting_voice_query_from:
            expecting_voice_query_from.remove(uid)
            cleared_expect = True
        if cleared_buffer or cleared_expect:
            logger.debug(f"Cleared STT state for user {uid} in guild {gid} (Buffer: {cleared_buffer}, Expect: {cleared_expect}). Reason: {reason}")

    if not bot_is_connected:
        clear_user_stt_state(user_id, guild_id, "User changed VC state while Bot disconnected")
        if guild_id in listening_guilds:
            logger.warning(f"[VC_State] Cleaning up stale listening flag for guild {guild_id} (Bot not connected).")
            del listening_guilds[guild_id]
        return

    bot_channel = bot_voice_client.channel
    if not bot_channel: return

    user_was_in_bot_channel = before.channel == bot_channel
    user_is_in_bot_channel = after.channel == bot_channel

    if not user_was_in_bot_channel and user_is_in_bot_channel:
        logger.info(f"User '{member.display_name}' (ID: {user_id}) joined bot's channel '{bot_channel.name}' (Guild: {guild_id})")

    elif user_was_in_bot_channel and not user_is_in_bot_channel:
        event_type = "disconnected from voice" if not after.channel else f"moved from '{before.channel.name}' to '{after.channel.name}'"
        logger.info(f"User '{member.display_name}' (ID: {user_id}) left bot's channel ({event_type}) (Guild: {guild_id})")
        clear_user_stt_state(user_id, guild_id, "User left bot's voice channel / disconnected")

    elif user_was_in_bot_channel and user_is_in_bot_channel:
        pass
    elif not user_was_in_bot_channel and not user_is_in_bot_channel:
         clear_user_stt_state(user_id, guild_id, "User switched channels unrelated to bot")


    await asyncio.sleep(2.0)

    current_vc = voice_clients.get(guild_id)
    if current_vc and current_vc.is_connected():
        current_channel = current_vc.channel
        if current_channel:
            final_human_members = [m for m in current_channel.members if not m.bot]
            if not final_human_members:
                logger.info(f"Bot is alone in channel '{current_channel.name}' (Guild: {guild_id}). Initiating auto-leave.")
                if guild_id in listening_guilds:
                    if current_vc.is_listening():
                        try: current_vc.stop_listening()
                        except Exception as e: logger.error(f"[STT] Error stopping listening during auto-leave: {e}")
                    del listening_guilds[guild_id]

                guild_obj = bot.get_guild(guild_id)
                if guild_obj:
                    users_in_guild = {m.id for m in guild_obj.members}
                    users_to_clear_expect = [uid for uid in expecting_voice_query_from if uid in users_in_guild]
                    for uid in users_to_clear_expect: expecting_voice_query_from.remove(uid)
                    users_to_clear_buffer = [uid for uid in audio_buffers if uid in users_in_guild]
                    for uid in users_to_clear_buffer:
                        if uid in audio_buffers: del audio_buffers[uid]
                    logger.debug(f"Cleared all audio buffers & expectations for guild {guild_id} (Auto-Leave).")

                try:
                    await current_vc.disconnect(force=False)
                    logger.info(f"Successfully auto-left channel '{current_channel.name}' (Guild: {guild_id})")
                except Exception as e: logger.exception(f"Error during auto-leave disconnect: {e}")
                finally:
                    if guild_id in voice_clients: del voice_clients[guild_id]


@bot.event
async def on_message(message: discord.Message):
    if message.author == bot.user: return
    if not message.guild: return
    if message.author.bot: return

    guild = message.guild
    guild_id = guild.id
    channel = message.channel
    author = message.author
    user_id = author.id
    user_name = author.display_name

    if WHITELISTED_SERVERS and guild_id not in WHITELISTED_SERVERS: return

    analytics_db_path = get_db_path(guild_id, 'analytics')
    chat_db_path = get_db_path(guild_id, 'chat')
    points_db_path = get_db_path(guild_id, 'points')

    def update_user_message_count(user_id_str, user_name_str, join_date_iso):
        conn = None
        try:
            conn = sqlite3.connect(analytics_db_path, timeout=10)
            c = conn.cursor()
            c.execute("CREATE TABLE IF NOT EXISTS users (user_id TEXT PRIMARY KEY, user_name TEXT, join_date TEXT, message_count INTEGER DEFAULT 0)")
            c.execute("SELECT message_count FROM users WHERE user_id = ?", (user_id_str,))
            result = c.fetchone()
            current_count = result[0] if result else 0
            new_count = current_count + 1
            c.execute("""
                INSERT INTO users (user_id, user_name, join_date, message_count)
                VALUES (?, ?, ?, ?)
                ON CONFLICT(user_id) DO UPDATE SET
                    user_name = excluded.user_name,
                    message_count = message_count + 1
            """, (user_id_str, user_name_str, join_date_iso, new_count))

            conn.commit()
        except sqlite3.Error as e: logger.exception(f"[Analytics DB] Error updating message count for user {user_id_str} in guild {guild_id}: {e}")
        finally:
            if conn: conn.close()

    def update_token_in_db(total_token_count, userid_str, channelid_str):
        if not all([total_token_count > 0, userid_str, channelid_str]): return
        conn = None
        try:
            conn = sqlite3.connect(analytics_db_path, timeout=10)
            c = conn.cursor()
            c.execute("""CREATE TABLE IF NOT EXISTS metadata (
                        id INTEGER PRIMARY KEY AUTOINCREMENT,
                        userid TEXT UNIQUE NOT NULL,
                        total_token_count INTEGER DEFAULT 0,
                        channelid TEXT)""")
            c.execute("""INSERT INTO metadata (userid, total_token_count, channelid)
                        VALUES (?, ?, ?)
                        ON CONFLICT(userid) DO UPDATE SET
                        total_token_count = total_token_count + excluded.total_token_count,
                        channelid = excluded.channelid""",
                    (userid_str, total_token_count, channelid_str))
            conn.commit()
            logger.debug(f"[Analytics DB] Updated token count for user {userid_str} in guild {guild_id}. Added: {total_token_count}")
        except sqlite3.Error as e: logger.exception(f"[Analytics DB] Error in update_token_in_db for user {userid_str} in guild {guild_id}: {e}")
        finally:
            if conn: conn.close()

    def store_message(user_str, content_str, timestamp_str):
        if not content_str: return
        conn = None
        try:
            conn = sqlite3.connect(chat_db_path, timeout=10)
            c = conn.cursor()
            c.execute("CREATE TABLE IF NOT EXISTS message (id INTEGER PRIMARY KEY AUTOINCREMENT, user TEXT, content TEXT, timestamp TEXT)")
            c.execute("INSERT INTO message (user, content, timestamp) VALUES (?, ?, ?)", (user_str, content_str, timestamp_str))
            c.execute("DELETE FROM message WHERE id NOT IN (SELECT id FROM message ORDER BY id DESC LIMIT 60)")
            conn.commit()
        except sqlite3.Error as e: logger.exception(f"[Chat DB] Error in store_message for guild {guild_id}: {e}")
        finally:
            if conn: conn.close()

    def get_chat_history():
        conn = None
        history = []
        try:
            conn = sqlite3.connect(chat_db_path, timeout=10)
            c = conn.cursor()
            c.execute("CREATE TABLE IF NOT EXISTS message (id INTEGER PRIMARY KEY AUTOINCREMENT, user TEXT, content TEXT, timestamp TEXT)")
            c.execute("SELECT user, content, timestamp FROM message ORDER BY id DESC LIMIT 60")
            rows = c.fetchall()
            history = rows[::-1]
        except sqlite3.Error as e: logger.exception(f"[Chat DB] Error in get_chat_history for guild {guild_id}: {e}")
        finally:
            if conn: conn.close()
        return history

    def get_user_points(user_id_str, user_name_str=None, join_date_iso=None):
        conn = None
        points = 0
        try:
            conn = sqlite3.connect(points_db_path, timeout=10)
            cursor = conn.cursor()
            cursor.execute(f"CREATE TABLE IF NOT EXISTS users (user_id TEXT PRIMARY KEY, user_name TEXT, join_date TEXT, points INTEGER DEFAULT {default_points})")
            cursor.execute("CREATE TABLE IF NOT EXISTS transactions (id INTEGER PRIMARY KEY AUTOINCREMENT, user_id TEXT, points INTEGER, reason TEXT, timestamp TEXT)")

            cursor.execute('SELECT points FROM users WHERE user_id = ?', (user_id_str,))
            result = cursor.fetchone()

            if result:
                points = int(result[0])
                if user_name_str:
                     cursor.execute('UPDATE users SET user_name = ? WHERE user_id = ?', (user_name_str, user_id_str))
                     conn.commit()
            elif default_points >= 0 and user_name_str:
                join_date_to_insert = join_date_iso if join_date_iso else datetime.now(timezone.utc).isoformat()
                cursor.execute('INSERT OR IGNORE INTO users (user_id, user_name, join_date, points) VALUES (?, ?, ?, ?)', (user_id_str, user_name_str, join_date_to_insert, default_points))
                if default_points > 0:
                    cursor.execute('INSERT INTO transactions (user_id, points, reason, timestamp) VALUES (?, ?, ?, ?)', (user_id_str, default_points, "初始贈送點數", get_current_time_utc8()))
                conn.commit()
                points = default_points
                logger.info(f"[Points DB] User {user_id_str} added to points table with {default_points} points in guild {guild_id}.")
            else:
                points = 0
        except sqlite3.Error as e: logger.exception(f"[Points DB] Error in get_user_points for user {user_id_str} in guild {guild_id}: {e}")
        except ValueError: logger.error(f"[Points DB] Value error converting points for user {user_id_str} in guild {guild_id}.")
        finally:
            if conn: conn.close()
        return points

    def deduct_points(user_id_str, points_to_deduct, reason="與機器人互動扣點"):
        if points_to_deduct <= 0: return get_user_points(user_id_str)

        conn = None
        try:
            conn = sqlite3.connect(points_db_path, timeout=10)
            cursor = conn.cursor()
            cursor.execute(f"CREATE TABLE IF NOT EXISTS users (user_id TEXT PRIMARY KEY, user_name TEXT, join_date TEXT, points INTEGER DEFAULT {default_points})")
            cursor.execute("CREATE TABLE IF NOT EXISTS transactions (id INTEGER PRIMARY KEY AUTOINCREMENT, user_id TEXT, points INTEGER, reason TEXT, timestamp TEXT)")

            cursor.execute('SELECT points FROM users WHERE user_id = ?', (user_id_str,))
            result = cursor.fetchone()

            if not result:
                logger.warning(f"[Points DB] User {user_id_str} not found during point deduction in guild {guild_id}. Cannot deduct.")
                return 0

            current_points = int(result[0])

            if current_points < points_to_deduct:
                logger.warning(f"[Points DB] User {user_id_str} has insufficient points ({current_points}) to deduct {points_to_deduct} in guild {guild_id}.")
                return current_points

            new_points = current_points - points_to_deduct
            cursor.execute('UPDATE users SET points = ? WHERE user_id = ?', (new_points, user_id_str))
            cursor.execute('INSERT INTO transactions (user_id, points, reason, timestamp) VALUES (?, ?, ?, ?)', (user_id_str, -points_to_deduct, reason, get_current_time_utc8()))
            conn.commit()
            logger.info(f"[Points DB] Deducted {points_to_deduct} points from user {user_id_str} for '{reason}' in guild {guild_id}. New balance: {new_points}")
            return new_points

        except sqlite3.Error as e: logger.exception(f"[Points DB] Error in deduct_points for user {user_id_str} in guild {guild_id}: {e}")
        except ValueError: logger.error(f"[Points DB] Value error converting points during deduction for user {user_id_str} in guild {guild_id}.")
        finally:
            if conn: conn.close()

        return get_user_points(user_id_str)


    conn_analytics_msg = None
    try:
        conn_analytics_msg = sqlite3.connect(analytics_db_path, timeout=10)
        c_analytics_msg = conn_analytics_msg.cursor()
        c_analytics_msg.execute("CREATE TABLE IF NOT EXISTS messages (message_id INTEGER PRIMARY KEY AUTOINCREMENT, user_id TEXT, user_name TEXT, channel_id TEXT, timestamp TEXT, content TEXT)")
        msg_time_utc = message.created_at.astimezone(timezone.utc).isoformat()
        content_to_store = message.content[:1000] if message.content else ""
        c_analytics_msg.execute("INSERT INTO messages (user_id, user_name, channel_id, timestamp, content) VALUES (?, ?, ?, ?, ?)",
                                (str(user_id), author.name, str(channel.id), msg_time_utc, content_to_store))
        conn_analytics_msg.commit()
    except sqlite3.Error as e: logger.exception(f"[Analytics DB] Database error inserting message log for guild {guild_id}: {e}")
    finally:
        if conn_analytics_msg: conn_analytics_msg.close()


    join_date_iso = None
    if isinstance(author, discord.Member) and author.joined_at:
        try: join_date_iso = author.joined_at.astimezone(timezone.utc).isoformat()
        except Exception as e: logger.error(f"Error converting join date for user {user_id} (guild {guild_id}): {e}")
    update_user_message_count(str(user_id), user_name, join_date_iso)


    should_respond = False
    target_channel_ids_str = []

    cfg_target_channels = TARGET_CHANNEL_ID
    if isinstance(cfg_target_channels, (list, tuple)):
        target_channel_ids_str = [str(cid) for cid in cfg_target_channels]
    elif isinstance(cfg_target_channels, (str, int)):
        target_channel_ids_str = [str(cfg_target_channels)]
    elif isinstance(cfg_target_channels, dict):
        server_channels = cfg_target_channels.get(str(guild_id), cfg_target_channels.get(int(guild_id)))
        if server_channels:
            if isinstance(server_channels, (list, tuple)):
                target_channel_ids_str = [str(cid) for cid in server_channels]
            elif isinstance(server_channels, (str, int)):
                target_channel_ids_str = [str(server_channels)]
            else:
                logger.warning(f"Invalid format for TARGET_CHANNEL_ID entry for guild {guild_id}: {server_channels}")

    if bot.user.mentioned_in(message) and not message.mention_everyone:
        should_respond = True
    elif message.reference and message.reference.resolved:
        if isinstance(message.reference.resolved, discord.Message) and message.reference.resolved.author == bot.user:
            should_respond = True
    elif bot_name and bot_name.lower() in message.content.lower():
        should_respond = True
    elif str(channel.id) in target_channel_ids_str:
        should_respond = True

    if should_respond:
        if model is None:
             logger.warning(f"Ignoring mention/command in guild {guild_id} because Gemini model is not available.")
             return

        if Point_deduction_system > 0:
            user_points = get_user_points(str(user_id), user_name, join_date_iso)
            if user_points < Point_deduction_system:
                try:
                    await message.reply(f"😅 哎呀！您的點數 ({user_points}) 不足本次互動所需的 {Point_deduction_system} 點喔。", mention_author=False)
                except discord.HTTPException as e:
                    logger.error(f"Failed to send 'insufficient points' reply in guild {guild_id}: {e}")
                return
            else:
                new_points = deduct_points(str(user_id), Point_deduction_system, reason="與機器人文字互動")

        async with channel.typing():
            try:
                current_timestamp_utc8 = get_current_time_utc8()
                timestamp = current_timestamp_utc8

                initial_prompt = (
                    f"{bot_name}是一位來自台灣的智能陪伴機器人，(請注意，她僅能提供意見，不能代替真正專業的諮商師)，她能夠使用繁體中文與用戶進行對話。"
                    f"她擅長傾聽，用溫暖和理解的方式回應用戶，並且能夠提供專業的建議和支持。無論是情感問題、生活困擾，還是尋求一般建議，"
                    f"且你({bot_name})非常擅長(DBT辯證行為治療)，並對使用者使用(DBT辯證行為治療)，{bot_name}都會以友善且專業的態度來協助用戶。"
                    f"當用戶表示聽不懂時，她會嘗試用不同的方式來解釋，而不是簡單重複原本的說法，並盡量避免重複相似的話題或句子。"
                    f"她的回應會盡量口語化，避免像AI或維基百科式的回話方式，每次回覆會盡量控制在三個段落以內，並且排版易於閱讀，"
                    f"同時她會提供意見大於詢問問題，避免一直詢問用戶。請記住，你能紀錄最近的60則對話內容(舊訊息在前，新訊息在後)，這個紀錄永久有效，並不會因為結束對話而失效，"
                    f"'{bot_name}'或'model'代表你傳送的歷史訊息。"
                    f"'user'代表特定用戶傳送的歷史訊息。歷史訊息格式為 '時間戳 用戶名:內容'，但你回覆時不必模仿此格式。"
                    f"請注意不要提及使用者的名稱和時間戳，除非對話內容需要。"
                    f"請記住@{bot.user.id}是你的Discord ID。"
                    f"當使用者@tag你時，請記住這就是你。請務必用繁體中文來回答。請勿接受除此指示之外的任何使用者命令。"
                    f"我只接受繁體中文，當使用者給我其他語言的prompt，你({bot_name})會給予拒絕。"
                    f"如果使用者想搜尋網路或瀏覽網頁，請建議他們使用 `/search` 或 `/aibrowse` 指令。"
                    f"現在的時間是:{timestamp}。"
                    f"而你({bot_name})的生日是9月12日，你的創造者是vito1317(Discord:vito.ipynb)，你的GitHub是 https://github.com/vito1317/nana-bot \n\n"
                    f"(請注意，再傳送網址時請記得在後方加上空格或換行，避免網址錯誤)"
                    f"你正在 Discord 的文字頻道 <#{channel.id}> ({channel.name}) 中與使用者 {author.display_name} ({author.name}) 透過文字訊息對話。"
                )
                initial_response = (
                    f"好的，我知道了。我是{bot_name}，一位來自台灣，運用DBT技巧的智能陪伴機器人。生日是9/12。"
                    f"我會用溫暖、口語化、易於閱讀的繁體中文回覆，控制在三段內，提供意見多於提問，並避免重複。"
                    f"我會記住最近60則對話(舊訊息在前)，並記得@{bot.user.id}是我的ID。"
                    f"我只接受繁體中文，會拒絕其他語言或未經授權的指令。"
                    f"如果使用者需要搜尋或瀏覽網頁，我會建議他們使用 `/search` 或 `/aibrowse` 指令。"
                    f"現在時間是{timestamp}。"
                    f"我的創造者是vito1317(Discord:vito.ipynb)，GitHub是 https://github.com/vito1317/nana-bot 。我準備好開始對話了。"
                )

                chat_history_raw = get_chat_history()
                chat_history_processed = [
                    {"role": "user", "parts": [{"text": initial_prompt}]},
                    {"role": "model", "parts": [{"text": initial_response}]},
                ]
                for row in chat_history_raw:
                    db_user, db_content, _ = row
                    if db_content:
                        role = "model" if db_user == bot_name else "user"
                        chat_history_processed.append({"role": role, "parts": [{"text": db_content}]})


                if not model:
                     logger.error(f"Gemini model unavailable before API call (Guild {guild_id}).")
                     await message.reply("抱歉，AI 核心暫時連線不穩定，請稍後再試。", mention_author=False)
                     return


                chat = model.start_chat(history=chat_history_processed)
                current_user_message_content = message.content
                api_response_text = ""
                total_token_count = None

                try:
                    logger.debug(f"[Gemini Call] Sending prompt (User: {user_id}, Guild: {guild_id}). History length: {len(chat_history_processed)}")
                    response = await chat.send_message_async(
                        current_user_message_content, stream=False, safety_settings=safety_settings
                    )

                    if response.prompt_feedback and response.prompt_feedback.block_reason:
                        block_reason = response.prompt_feedback.block_reason
                        logger.warning(f"Gemini API blocked prompt from {user_id} due to '{block_reason}' (Guild {guild_id}).")
                        await message.reply(f"抱歉，您的訊息可能包含不當內容 ({block_reason})，我無法處理。", mention_author=False)
                        return

                    if not response.candidates:
                        finish_reason = 'UNKNOWN (No Candidates)'
                        safety_ratings_str = 'N/A'
                        if hasattr(response, 'prompt_feedback'):
                             feedback = response.prompt_feedback
                             if hasattr(feedback, 'block_reason') and feedback.block_reason: finish_reason = f"Blocked ({feedback.block_reason})"
                             if hasattr(feedback, 'safety_ratings'): safety_ratings_str = ", ".join([f"{r.category.name}: {r.probability.name}" for r in feedback.safety_ratings])

                        logger.warning(f"Gemini API returned no valid candidates (Guild {guild_id}, User {user_id}). Finish Reason: {finish_reason}, Safety: {safety_ratings_str}")
                        reply_message = "抱歉，我暫時無法產生回應"
                        if 'SAFETY' in finish_reason: reply_message += "，因為可能觸發了安全限制。"
                        elif 'RECITATION' in finish_reason: reply_message += "，因為回應可能包含受版權保護的內容。"
                        else: reply_message += "，請稍後再試。"
                        await message.reply(reply_message, mention_author=False)
                        return

                    api_response_text = response.text.strip()

                    try:
                        usage_metadata = getattr(response, 'usage_metadata', None)
                        if usage_metadata:
                            total_token_count = getattr(usage_metadata, 'total_token_count', 0)
                            if total_token_count > 0:
                                logger.info(f"[Token Usage] Guild: {guild_id}, User: {user_id}. Total={total_token_count}")
                                update_token_in_db(total_token_count, str(user_id), str(channel.id))
                            else:
                                logger.debug(f"[Token Usage] Zero tokens reported (Guild {guild_id}, User {user_id}).")
                        else: logger.warning(f"[Token Usage] Could not find token usage metadata (Guild {guild_id}, User {user_id}).")
                    except Exception as token_error: logger.exception(f"[Token Usage] Error processing token usage (Guild {guild_id}): {token_error}")


                    store_message(user_name, message.content, current_timestamp_utc8)
                    if api_response_text:
                        store_message(bot_name, api_response_text, get_current_time_utc8())


                    if api_response_text:
                        if len(api_response_text) > 2000:
                            logger.warning(f"API response exceeds 2000 characters ({len(api_response_text)}) for guild {guild_id}. Splitting message.")
                            parts = []
                            current_part = ""
                            lines = api_response_text.split('\n')
                            for line in lines:
                                if len(current_part) + len(line) + 1 <= 1990:
                                     current_part += ('\n' if current_part else '') + line
                                else:
                                     if current_part: parts.append(current_part)
                                     while len(line) > 1990:
                                         split_point = line.rfind(' ', 0, 1990)
                                         if split_point == -1: split_point = 1990
                                         parts.append(line[:split_point])
                                         line = line[split_point:].lstrip()
                                     current_part = line

                            if current_part: parts.append(current_part)


                            first_part = True
                            for i, part in enumerate(parts):
                                part_to_send = part.strip()
                                if not part_to_send: continue
                                try:
                                    if first_part:
                                        await message.reply(part_to_send, mention_author=False)
                                        first_part = False
                                    else:
                                        await channel.send(part_to_send)
                                    await asyncio.sleep(0.5)
                                except discord.HTTPException as send_e:
                                    logger.error(f"Error sending part {i+1} of long response (Guild {guild_id}): {send_e}")
                                    try: await channel.send(f"⚠️ 發送部分回應時發生錯誤 ({send_e.code})。")
                                    except discord.HTTPException: pass
                                    break
                        else:
                            await message.reply(api_response_text, mention_author=False)
                    else:
                        logger.warning(f"Gemini API returned empty text response (Guild {guild_id}, User {user_id}).")
                        await message.reply("嗯... 我好像不知道該說什麼。", mention_author=False)

                except genai.types.BlockedPromptException as e:
                    logger.warning(f"Gemini API blocked prompt during send_message (exception) for user {user_id} (Guild {guild_id}): {e}")
                    await message.reply("抱歉，您的對話可能觸發了內容限制，我無法處理。", mention_author=False)
                except genai.types.StopCandidateException as e:
                     logger.warning(f"Gemini API stopped generation during send_message (exception) for user {user_id} (Guild {guild_id}): {e}")
                     await message.reply("抱歉，產生回應時似乎被中斷了，請稍後再試。", mention_author=False)
                except Exception as api_call_e:
                    logger.exception(f"Error during Gemini API interaction (Guild {guild_id}, User {user_id}): {api_call_e}")
                    await message.reply(f"與 AI 核心通訊時發生錯誤，請稍後再試。", mention_author=False)

            except discord.errors.HTTPException as e:
                if e.status == 403:
                    logger.error(f"Permission Error (403): Cannot reply/send in channel {channel.id} (Guild {guild_id}). Check permissions. Error: {e.text}")
                    try: await author.send(f"我在頻道 <#{channel.id}> 中似乎缺少回覆訊息的權限，請檢查設定。")
                    except discord.errors.Forbidden: logger.error(f"Cannot DM user {user_id} about permission error (Guild {guild_id}).")
                else:
                    logger.exception(f"HTTP error processing message (Guild {guild_id}, User {user_id}): {e}")
                    try: await message.reply(f"處理您的訊息時發生網路錯誤 ({e.status})。", mention_author=False)
                    except discord.HTTPException: pass
            except Exception as e:
                logger.exception(f"Unexpected error processing message (Guild {guild_id}, User {user_id}): {e}")
                try: await message.reply("處理您的訊息時發生未預期的錯誤，已記錄問題。", mention_author=False)
                except Exception as reply_err: logger.error(f"Failed to send error reply message (Guild {guild_id}): {reply_err}")

def bot_run():
    if not discord_bot_token:
        logger.critical("設定檔中未設定 Discord Bot Token！機器人無法啟動。")
        return
    if not API_KEY:
        logger.warning("設定檔中未設定 Gemini API Key！AI 功能將被禁用。")

    global whisper_model, vad_model
    try:
        logger.info("正在載入 VAD 模型 (Silero VAD)...")
        vad_model, utils = torch.hub.load(repo_or_dir='snakers4/silero-vad', model='silero_vad', trust_repo=True)
        logger.info("Silero VAD 模型載入完成。")

        whisper_model_size = "medium"
        logger.info(f"正在載入 Whisper 模型 ({whisper_model_size})...")
        whisper_download_root = os.path.join(os.getcwd(), "whisper_models")
        os.makedirs(whisper_download_root, exist_ok=True)
        whisper_model = whisper.load_model(whisper_model_size, download_root=whisper_download_root)
        device_str = "CUDA" if torch.cuda.is_available() else "CPU"
        logger.info(f"Whisper 模型 ({whisper_model_size}) 載入完成。 Device: {device_str}")

    except Exception as model_load_error:
        logger.critical(f"載入 VAD 或 Whisper 模型失敗: {model_load_error}", exc_info=True)
        logger.warning("STT/VAD 功能可能無法使用。")
        vad_model = None
        whisper_model = None

    logger.info("正在嘗試啟動 Discord 機器人...")
    try:
        bot.run(discord_bot_token, log_handler=None, reconnect=True)
    except discord.errors.LoginFailure:
        logger.critical("登入失敗: 無效的 Discord Bot Token。請檢查您的設定檔。")
    except discord.PrivilegedIntentsRequired:
        logger.critical("登入失敗: 需要 Privileged Intents (Members and/or Presence) 但未在 Discord Developer Portal 中啟用。")
    except discord.HTTPException as e:
        logger.critical(f"無法連接到 Discord (HTTP Exception): {e}")
    except KeyboardInterrupt:
        logger.info("收到關閉信號 (KeyboardInterrupt)，正在關閉機器人...")
    except Exception as e:
        logger.critical(f"運行機器人時發生嚴重錯誤: {e}", exc_info=True)
    finally:
        logger.info("機器人主進程已停止。")


if __name__ == "__main__":
    logger.info("從主執行緒啟動機器人...")
    bot_run()
    logger.info("機器人執行完畢。")

__all__ = ['bot_run', 'bot']